/*
 * This file is part of the Polyglot extensible compiler framework.
 *
 * Copyright (c) 2000-2006 Polyglot project group, Cornell University
 * Copyright (c) 2006 IBM Corporation
 * 
 */

package polyglot.parse;

import polyglot.ast.*;
import polyglot.types.TypeSystem;
import polyglot.util.*;

/**
 * Represents an ambiguous, possibly qualified, identifier encountered while parsing.
 */
public class Name {
    public final Name prefix;
    public final Id name;
    public final Position pos;
    public final NodeFactory nf;
    public final TypeSystem ts;

    public Name(NodeFactory nf, TypeSystem ts, Position pos, Id name) {
        this(nf, ts, pos, null, name);
    }
    
    public Name(NodeFactory nf, TypeSystem ts, Position pos, Name prefix, Id name) {
        this.nf = nf;
        this.ts = ts;
        this.pos = pos != null ? pos : Position.COMPILER_GENERATED;
        this.prefix = prefix;
        this.name = name;
    }

    /** @deprecated */
    private Name(NodeFactory nf, TypeSystem ts, Position pos, Name prefix, String qualifiedName) {
    	this.nf = nf;
        this.ts = ts;
        this.pos = pos != null ? pos : Position.COMPILER_GENERATED;
        
        if (! StringUtil.isNameShort(qualifiedName)) {
            if (prefix == null) {
                Position prefixPos = pos.truncateEnd(qualifiedName.length()+1);
                Position namePos = new Position(pos.truncateEnd(qualifiedName.length()).endOf(), pos.endOf());
                this.prefix = new Name(nf, ts, prefixPos, null, StringUtil.getPackageComponent(qualifiedName));
                this.name = nf.Id(namePos, StringUtil.getShortNameComponent(qualifiedName));
            }
            else {
                throw new InternalCompilerError("Can only construct a qualified Name with a short name string: " + qualifiedName + " is not short.");
            }
        }
        else {
        	Position idPos;
        	
            if (prefix == null) {
              idPos = pos;
            }
            else {
             idPos = new Position(pos.truncateEnd(qualifiedName.length()).endOf(), pos.endOf());
            }
            this.prefix = prefix;
            this.name = nf.Id(idPos, qualifiedName);
        }
    }
    
    // expr
    public Expr toExpr() {
        if (prefix == null) {
            return nf.AmbExpr(pos, name);
        }

        return nf.Field(pos, prefix.toReceiver(), name);
    }

    // expr or type
    public Receiver toReceiver() {
        if (prefix == null) {
            return nf.AmbReceiver(pos, name);
        }

        return nf.AmbReceiver(pos, prefix.toPrefix(), name);
    }

    // expr, type, or package
    public Prefix toPrefix() {
        if (prefix == null) {
            return nf.AmbPrefix(pos, name);
        }

        return nf.AmbPrefix(pos, prefix.toPrefix(), name);
    }

    // type or package
    public QualifierNode toQualifier() {
        if (prefix == null) {
            return nf.AmbQualifierNode(pos, name);
        }

        return nf.AmbQualifierNode(pos, prefix.toQualifier(), name);
    }

    // package
    public PackageNode toPackage() {
        if (prefix == null) {
            return nf.PackageNode(pos, ts.createPackage(null, name.id()));
        }
        else {
            return nf.PackageNode(pos, ts.createPackage(prefix.toPackage().package_(), name.id()));
        }
    }

    // type
    public TypeNode toType() {
        if (prefix == null) {
            return nf.AmbTypeNode(pos, name);
        }

        return nf.AmbTypeNode(pos, prefix.toQualifier(), name);
    }

    public String toString() {
        if (prefix == null) {
            return name.toString();
        }

        return prefix.toString() + "." + name.toString();
    }
}
